--[[
SimpleHalo
Author: Michael Joseph Murray aka Lyte of Lothar(US)
$Revision: 250 $
$Date: 2012-10-11 20:27:15 +0000 (Thu, 11 Oct 2012) $
Project Version: r250
contact: codemaster2010 AT gmail DOT com

Copyright (c) 2012 Michael J. Murray aka Lyte of Lothar(US)
All rights reserved unless otherwise explicitly stated.
]]

SimpleHalo = LibStub("AceAddon-3.0"):NewAddon("SimpleHalo", "AceConsole-3.0", "AceEvent-3.0")
local LSM = LibStub("LibSharedMedia-3.0")
local Range = LibStub("LibRangeCheck-2.0")

function SimpleHalo:OnInitialize()
	local defaults = {
		profile = {
			position = {},
			locked = false,
		},
	}
	
	self.db = LibStub("AceDB-3.0"):New("SimpleHaloDB", defaults, "Default")
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Refresh")
	self.db.RegisterCallback(self, "OnProfileChanged", "Refresh")
	
	self:RegisterChatCommand("halo", "OpenConfig", true, true)
	self:RegisterChatCommand("simplehalo", "OpenConfig")
	
	self.indicator = self:CreateIndicator()
end

function SimpleHalo:OnEnable()
	self:RegisterEvent("PLAYER_REGEN_ENABLED", "LeaveCombat")
	self:RegisterEvent("PLAYER_REGEN_DISABLED", "EnterCombat")
end

function SimpleHalo:OnDisable()

end

function SimpleHalo:Reset()
	self.indicator:ClearAllPoints()
	self.indicator:SetPoint("CENTER", UIParent, "CENTER")
end

function SimpleHalo:Refresh()
	local db = self.db.profile
	self.indicator:ClearAllPoints()
	if db.position.x then
		self.inticator:SetPoint(db.position.point, UIParent, db.position.anchor, db.position.x, db.position.y)
	else
		self.indicator:SetPoint("CENTER", UIParent, "CENTER")
	end
end

function SimpleHalo:OpenConfig()
	--LibStub("AceConfigDialog-3.0"):Open("SimpleHalo")
end

function SimpleHalo:EnterCombat()
	--show the frame and start range polling
	self.indicator:Show()
end

function SimpleHalo:LeaveCombat()
	--hide the frame to stop range polling
	self.indicator:Hide()
end

local lastUpdate = 0
local function updateRange(frame, elapsed)
	lastUpdate = lastUpdate + elapsed
	if lastUpdate < 0.15 then return end
	
	lastUpdate = 0
	
	local minRange, maxRange = Range:GetRange("target")
	
	--This is the same logic used by HaloPro, which the Theorycrafters love
	if (not minRange) or (not maxRange) then
		--no range information
		frame.icon:SetVertexColor(1,1,1)
		frame.text:SetText("")
	elseif maxRange <= 15 then
		--within 15 yards of target
		--too close!
		frame.icon:SetVertexColor(1,0,0)
		frame.text:SetText("-")
	elseif minRange >= 15 and maxRange <= 20 then
		--between 15 and 20 yards of target
		--slightly too close
		frame.icon:SetVertexColor(1,1,0)
		frame.text:SetText("-")
	elseif minRange >= 20 and maxRange <= 25 then
		--sweet spot
		frame.icon:SetVertexColor(0,1,0)
		frame.text:SetText("")
	elseif minRange >= 25 and maxRange <=30 then
		--between 25 and 30 yards of target
		--slightly too far
		frame.icon:SetVertexColor(1,1,0)
		frame.text:SetText("+")
	elseif minRange > 30 then
		--more than 30 yards
		--too far!
		frame.icon:SetVertexColor(1,0,0)
		frame.text:SetText("+")
	end
end

function SimpleHalo:CreateIndicator()
	local icon = [[Interface\AddOns\SimpleHalo\icon.tga]]
	local f = CreateFrame("FRAME", nil, UIParent)
	
	--size and positon
	f:SetWidth(100)
	f:SetHeight(50)
	
	if self.db.profile.position.x and self.db.profile.position.y and self.db.profile.position.anchor and self.db.profile.position.point then
		f:ClearAllPoints()
		f:SetPoint(self.db.profile.position.point, UIParent, self.db.profile.position.anchor, self.db.profile.position.x, self.db.profile.position.y)
	else
		f:SetPoint("CENTER", UIParent, "CENTER")
	end
	
	--style the frame
	f:SetBackdrop({
		bgFile = [[Interface\Tooltips\UI-Tooltip-Background]],
		edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]],
		tile = false, tileSize = 16, edgeSize = 16,
		insets = { left = 5, right = 5, top = 5, bottom = 5 }
	})
	f:SetBackdropColor(0.65, 0.65, 0.65, 0.65)
	
	f.icon = f:CreateTexture()
	f.icon:SetTexture(icon)
	f.icon:SetHeight(25)
	f.icon:SetWidth(25)
	f.icon:ClearAllPoints()
	f.icon:SetPoint("LEFT", f, "LEFT", 5, 0)

	f.text = f:CreateFontString()
	f.text:SetHeight(25)
	f.text:SetWidth(25)
	f.text:SetFont([[Fonts\FRIZQT__.TTF]], 16)
	f.text:SetTextColor(1, 1, 1, 1)
	f.text:SetJustifyH("LEFT")
	f.text:SetJustifyV("MIDDLE")
	f.text:ClearAllPoints()
	f.text:SetPoint("RIGHT", f, "RIGHT", -5, 0)
	
	--setup dragging
	f:SetClampedToScreen(true)
	f:SetMovable(true)
	f:RegisterForDrag("LeftButton")
	if self.db.profile.locked then
		f:EnableMouse(false)
	else
		f:EnableMouse(true)
	end
	
	f:SetScript("OnDragStart", function() f:StartMoving() end )
	f:SetScript("OnDragStop", function ()
		f:StopMovingOrSizing()
		local point, _, anchor, x, y = f:GetPoint()
		SimpleHalo.db.profile.position.x = floor(x)
		SimpleHalo.db.profile.position.y = floor(y)
		SimpleHalo.db.profile.position.anchor = anchor
		SimpleHalo.db.profile.position.point = point
	end)
	
	--set our update script
	f:SetScript("OnUpdate", updateRange)
	
	--hide and return
	f:Hide()
	return f
end
